********************************************************************
**             one_dim.for
********************************************************************
********************************************************************
** This program is a main code that computes the price of an 
** integrated variance option at time t=0 when the asset dynamics 
** is given by the Heston model
*********************************************************************
      implicit none
      integer n1
      parameter(n1=2048*2048)
	complex*16 u(0:n1),ff(0:n1),dfft,xi
	real*8  xiv(0:n1),nris
      real*8 rint,vero,chi
      integer j,jj, j1,j2,j3,jx,jv,ji,mm,k
      complex*16 ui,beta,zita,mu,gamma,csum
	real*8 theta,s,rho,tau,eps,kchi,pi,pi2,v0
	real*8 x0
	real*8 ss,dstep
      integer ix,iv,ii,it,N	
	real*8 x,xp,v,vp,i0,ip,i
	real*8  eprice,payoff
	complex*16 contrib
	real*8 option, rtax
      real*4 timep
	real*4 secnds
	external secnds 
	open(4,file='one_dim.txt',status='unknown')

	pi=dacos(-1.d0)
	pi2=2.d0*pi
	ui=dcmplx(0.d0,1.d0)


***********************************************************************************************
** The oscillatory integral (30) is computed using the method proposed by Bailey, Swarztrauber
** in the paper published on Siam Journal on Scientific Computing Vol 15(5) 1994 p. 1105-1110
***********************************************************************************************

**********************************************************************************************
** dstep: real number, generally a power of two that must be assigned
**        to determine the grid  of integration. Hint: dstep=256 or 512 (dstep<=2048)
**********************************************************************************************

	dstep=256.d0

	nris=dsqrt(pi2)/dstep

	mm=int(pi2/nris**2)


******************************************
**  Definition of the integration grid  **  
******************************************


       do j=0,mm-1
	  xiv(j)=dfloat(j-mm/2)*nris
      enddo


**********************************************
** Assign the parameters of the Heston model
**
** eps= parameter epsilon appearing in formulae (5) and (14)
** chi= parameter chi appearing in formuale (5) and (14)
** theta= parameter theta appearing in formuale (5) and (14)
*************************************************************

*************************************
* Parameters first experiment
*************************************
	eps=0.1d0
	chi=2.d0
	theta=0.01d0
	rho=-0.5

**********************************
** Parameters second experiment
**********************************
	eps=0.2d0
	chi=0.8d0
	theta=eps**2/(2.d0*chi)+0.1d0
      rho=0.1d0



**********************************************************
**   i0: initial integrated variance, i0=0 equation (10)
**   x0: initial centrated log-returns x0=0 equation (16)
**********************************************************
	i0=0.0d0
      x0=0.d0




*****************************************************************
*****************************************************************
*****************************************************************
*****************************************************************
**
** Parameters that must be assigned by the user
**
*****************************************************************
*****************************************************************

******************************************************************
** Assign: v0, eprice, tau, rtax
******************************************************************
******************************************************************
**     v0: initial volatility equation (17) 
** eprice: integrated variance strike price (that is the strike
**         price of the option on the realized variance) 
**    tau: time to maturity 
**   rtax: risk free interest rate 
******************************************************************
      v0=2.0d0
      eprice=0.5d0
      tau=1.d0
	rtax=0.d0


	s=2.d0*chi*theta/eps**2-1.d0

****************************************
** Note that s must be greater than zero
****************************************
	write(*,*) 'this paprameter must be greater than zero->',s
	write(*,*) 'wait please'

	ss=s+1
	kchi=chi-eps*0.5d0*rho
       
*************************************************
** Start integration procedure
*************************************************
      timep=secnds(0.0)

      do j=0,mm-1
	   xi=xiv(j) 
         zita=0.5d0*cdsqrt(chi**2+2.d0*eps**2*xi*ui)
         beta=chi*0.5+zita+(zita-0.5*chi)*cdexp(-2.d0*tau*zita)
         gamma=1.d0-cdexp(-2.d0*tau*zita)
	   ff(j)=cdexp(-ss*(-0.5*chi+zita)*tau+ss*cdlog(2.d0*zita/beta)
     &         -v0*ui*xi*gamma/beta)

         if(cdabs(xi).gt.1.d-6) then
            contrib=(-eprice)/(ui*xi)
   	      contrib=contrib+(cdexp(ui*xi*eprice)-1.d0)
     &/     ((ui*xi)*(ui*xi))
	   else
	      contrib=(eprice)*(eprice)*0.5d0
	   endif
	   ff(j)=ff(j)*contrib 
      enddo

	csum=(0.d0,0.d0)
	do j=0,mm-1
	   csum=csum+(-1.d0)**j*ff(j)
     &     *cdexp(-2.d0*pi*dfloat(mm*j)*0.5*ui/dfloat(mm))
	enddo	
      csum=csum*(-1.d0)**(mm*0.5)*nris/pi2
      vero=i0-eprice+theta*tau+(1.d0-dexp(-chi*tau))*(v0-theta)/chi
      csum=vero+csum

*****************************************
*** Compute the discounted value
*****************************************
      option=dexp(-rtax*tau)*dreal(csum)

*********************************************
** Compute the execution time of this program
*********************************************
	timep=secnds(timep)
	write(*,*)  'time=',timep
      write(*,*) 'option value=',option
	write(*,*) 'imaginary part',dimag(csum)

************************************************
** Write in the file one_dim.txt
************************************************
	write(4,*) 'time=',timep
      write(4,*) 'option value=',option
	write(4,*) 'imaginary part',dimag(csum)   

	end
