********************************************************************
**      two_dim.for
********************************************************************
********************************************************************
** This program is a main code that computes the price of an 
** integrated variance option at time t=0 with a generic payoff 
** when the asset dynamics is given by the Heston model
*********************************************************************
      implicit none
      integer n1
      parameter(n1=256*256)
	complex*16 u(0:n1),ff(0:n1),dfft,xi
	real*8 ivet(0:n1), xiv(0:n1),nris
      real*8 rtax,sumr,sumi,option
      integer j,jj, j1,j2,j3,jx,jv,ji,mm,k
      complex*16 ui,beta,zita,mu,gamma
	real*8 chi,theta,s,rho,tau,eps,kchi,pi,pi2,v0
	real*8 x0,x,xp,v,vp,i0,ip
	real*8  eprice,payoff,dstep,ss
	real*4 timep
	real*4 secnds

	open(4,file='two_dim.txt',status='unknown')
	
	pi=dacos(-1.d0)
	pi2=2.d0*pi
	ui=dcmplx(0.d0,1.d0)



***********************************************************************************************
** The oscillatory integral (28) is computed using the method proposed by Bailey, Swarztrauber
** in the paper published on Siam Journal on Scientific Computing Vol 15(5) 1994 p. 1105-1110
***********************************************************************************************

*****************************************************
** dstep: real number, generally a power of two that must to be assigned
**        to determine the grid  of integration. Hint: dstep=32, 64 (dstep<=256)
*************************************************************************
      dstep=16.d0
	nris=dsqrt(pi2)/dstep
	mm=int(pi2/nris**2)
	

******************************************
**  Definition of the integration grid  **  
******************************************
      do j=0,mm-1
	  xiv(j)=dfloat(j-mm/2)*nris
  	  ivet(j)=dfloat(j-mm/2)*pi2/(dfloat(mm)*nris)
	write(*,*) j,ivet(j)
      enddo


**********************************************
** Assign the parameters of the Heston model
**
** eps= parameter epsilon appearing in formulae (5) and (14)
** chi= parameter chi appearing in formuale (5) and (14)
** theta= parameter theta appearing in formuale (5) and (14)
*************************************************************

*************************************
* Parameters first experiment
*************************************
	eps=0.1d0
	chi=2.d0
	theta=0.01d0
	rho=-0.5


**********************************
** Parameters second experiment
**********************************
	eps=0.2d0
	chi=0.8d0
	theta=eps**2/(2.d0*chi)+0.1d0
      rho=0.1d0


**********************************************************
**   i0: initial integrated variance, i0=0 equation (10)
**   x0: initial centrated log-returns x0=0 equation (16)
**********************************************************
	i0=0.0d0
      x0=0.d0


*****************************************************************
*****************************************************************
*****************************************************************
*****************************************************************
**
** Parameters that must be assigned by the user
**
*****************************************************************
*****************************************************************

******************************************************************
** Assign: v0, eprice, tau, rtax
******************************************************************
******************************************************************
**     v0: initial volatility equation (17) 
** eprice: integrated variance strike price (that is the strike
**         price of the option on the realized variance) 
**    tau: time to maturity 
**   rtax: risk free interest rate 
******************************************************************
      v0=1.5d0
      eprice=0.5d0
      tau=1.d0
	rtax=0.d0

	s=2.d0*chi*theta/eps**2-1.d0

****************************************
** Note that s must be greater than zero
****************************************
	write(*,*) 'this paprameter must be greater than zero->',s
	write(*,*) 'wait please'
	
	
	ss=s+1
	kchi=chi-eps*0.5d0*rho
       

*************************************************
** Start integration procedure
*************************************************
      timep=secnds(0.0)

      do j=0,mm-1
	   xi=xiv(j) 
         zita=0.5d0*cdsqrt(chi**2+2.d0*eps**2*xi*ui)
         beta=chi*0.5+zita+(zita-0.5*chi)*cdexp(-2.d0*tau*zita)
         gamma=1.d0-cdexp(-2.d0*tau*zita)
	   ff(j)=cdexp(-ss*(-0.5*chi+zita)*tau+ss*cdlog(2.d0*zita/beta)
     &-v0*ui*xi*gamma/beta)
      enddo

	sumr=0.d0
	sumi=0.d0
      do k=0, mm-1
         ip=i0-ivet(k)
***********************************************************
*** In this part of the program the user must introduce
*** the computation of the payoff function. In the example
*** considered here the payoff is P(ip)=max{ip-eprice,0}.
************************************************************
	   payoff=ip-eprice
	   if(payoff.lt.0.d0) payoff=0.d0
  	   u(k)=((-1.d0)**k*nris*dfft(k,mm,ff))/pi2
	write(*,*) 'k=',k,'passo=',dreal(u(k))
	   sumr=sumr+dreal(u(k))*payoff
	   sumi=sumi+dimag(u(k))*payoff
     	enddo
      sumr=sumr*nris
	sumi=sumi*nris

*******************************************
*** Compute the discounted value
*****************************************
      option=dexp(-rtax*tau)*sumr

*********************************************
** Compute the execution time of this program
*********************************************
	timep=secnds(timep)
	write(*,*)  'time=',timep
      write(*,*) 'option value=',option
	write(*,*) 'imaginary part',sumi   
************************************************
** Write in the file two_dim.txt
************************************************
	write(4,*) 'time=',timep
      write(4,*) 'option value=',option
	write(4,*) 'imaginary part',sumi   

	end



*******************************************************
*** Subroutine that computes the data for the FFT     *
*******************************************************
      complex*16 function dfft(k,mm,ff)
	integer k,mm
	complex*16 ff(0:mm)
	complex*16 ui,csum,z,dxi
	real*8 pi
	pi=dacos(-1.d0)
	ui=dcmplx(0.d0,1.d0)
	dxi=2.d0*pi*dfloat(k)*ui/dfloat(mm)
	csum=(0.d0,0.d0)
	do j=0,mm-1
	   z=dxi*dfloat(j)
	   csum=csum+(-1.d0)**j*ff(j)*cdexp(-z)
	enddo
	dfft=csum
	return
	end
